@@ -15,6 +15,7 @@ from circus.watcher import Watcher
 from circus.util import debuglog, _setproctitle
 from circus.config import get_config
 from circus.plugins import get_plugin_cmd
+from circus.sockets import CircusSocket, CircusSockets
 
 
 class Arbiter(object):
@@ -39,10 +40,19 @@ class Arbiter(object):
 
         - **use** -- Fully qualified name that points to the plugin class
         - every other value is passed to the plugin in the **config** option
+    - **sockets** -- a mapping of sockets. Each key is the socket name,
+      and each value a :class:`CircusSocket` class. (default: None)
+    - **warmup_delay** -- a delay in seconds between two watchers startup.
+      (default: 0)
+    - **httpd** -- If True, a circushttpd process is run (default: False)
+    - **httpd_host** -- the circushttpd host (default: localhost)
+    - **httpd_port** -- the circushttpd port (default: 8080)
     """
     def __init__(self, watchers, endpoint, pubsub_endpoint, check_delay=1.,
                  prereload_fn=None, context=None, loop=None,
-                 stats_endpoint=None, plugins=None):
+                 stats_endpoint=None, plugins=None, sockets=None,
+                 warmup_delay=0, httpd=False, httpd_host='localhost',
+                 httpd_port=8080):
         self.watchers = watchers
         self.endpoint = endpoint
         self.check_delay = check_delay
@@ -68,9 +78,27 @@ class Arbiter(object):
             cmd += ' --endpoint %s' % self.endpoint
             cmd += ' --pubsub %s' % self.pubsub_endpoint
             cmd += ' --statspoint %s' % self.stats_endpoint
-            stats_watcher = Watcher('circusd-stats', cmd)
+            stats_watcher = Watcher('circusd-stats', cmd, use_sockets=True)
             self.watchers.append(stats_watcher)
 
+        # adding the httpd
+        if httpd:
+            cmd = ("%s -c 'from circus.web import circushttpd; "
+                   "circushttpd.main()'") % sys.executable
+            cmd += ' --endpoint %s' % self.endpoint
+            cmd += ' --fd $(circus.sockets.circushttpd)'
+
+            httpd_watcher = Watcher('circushttpd', cmd, use_sockets=True)
+            self.watchers.append(httpd_watcher)
+            httpd_socket = CircusSocket(name='circushttpd', host=httpd_host,
+                                        port=httpd_port)
+
+            # adding the socket
+            if sockets is None:
+                sockets = [httpd_socket]
+            else:
+                sockets.append(httpd_socket)
+
         # adding each plugin as a watcher
         if plugins is not None:
             for plugin in plugins:
@@ -81,6 +109,9 @@ class Arbiter(object):
                 plugin_watcher = Watcher(name, cmd, priority=1, singleton=True)
                 self.watchers.append(plugin_watcher)
 
+        self.sockets = CircusSockets(sockets)
+        self.warmup_delay = warmup_delay
+
     @classmethod
     def load_from_config(cls, config_file):
         cfg = get_config(config_file)
@@ -92,12 +123,20 @@ class Arbiter(object):
         for watcher in cfg.get('watchers', []):
             watchers.append(Watcher.load_from_config(watcher))
 
+        sockets = []
+        for socket in cfg.get('sockets', []):
+            sockets.append(CircusSocket.load_from_config(socket))
+
         # creating arbiter
         arbiter = cls(watchers, cfg['endpoint'], cfg['pubsub_endpoint'],
                       check_delay=cfg.get('check_delay', 1.),
                       prereload_fn=cfg.get('prereload_fn'),
                       stats_endpoint=cfg.get('stats_endpoint'),
-                      plugins=cfg.get('plugins'))
+                      plugins=cfg.get('plugins'), sockets=sockets,
+                      warmup_delay=cfg.get('warmup_delay', 0),
+                      httpd=cfg.get('httpd', False),
+                      httpd_host=cfg.get('httpd_host', 'localhost'),
+                      httpd_port=cfg.get('httpd_port', 8080))
 
         return arbiter
 
@@ -118,10 +157,15 @@ class Arbiter(object):
         self.evpub_socket.bind(self.pubsub_endpoint)
         self.evpub_socket.linger = 0
 
+        # initialize sockets
+        if len(self.sockets) > 0:
+            self.sockets.bind_and_listen_all()
+            logger.info("sockets started")
+
         # initialize watchers
         for watcher in self.iter_watchers():
             self._watchers_names[watcher.name.lower()] = watcher
-            watcher.initialize(self.evpub_socket)
+            watcher.initialize(self.evpub_socket, self.sockets)
 
     @debuglog
     def start(self):
@@ -142,6 +186,7 @@ class Arbiter(object):
         logger.debug('Initializing watchers')
         for watcher in self.iter_watchers():
             watcher.start()
+            time.sleep(self.warmup_delay)
 
         logger.info('Arbiter now waiting for commands')
         while True:
@@ -164,24 +209,28 @@ class Arbiter(object):
 
         self.loop.stop()
 
+        # close sockets
+        self.sockets.close_all()
+
     def reap_processes(self):
         # map watcher to pids
         watchers_pids = {}
         for watcher in self.iter_watchers():
             if not watcher.stopped:
-                for pid, wid in watcher.pids.items():
-                    watchers_pids[pid] = (watcher, wid)
+                for process in watcher.processes.values():
+                    watchers_pids[process.pid] = watcher
 
         # detect dead children
         while True:
             try:
+                # wait for our child (so it's not a zombie)
                 pid, status = os.waitpid(-1, os.WNOHANG)
                 if not pid:
                     break
 
                 if pid in watchers_pids:
-                    watcher, wid = watchers_pids[pid]
-                    watcher.reap_process(wid, status)
+                    watcher = watchers_pids[pid]
+                    watcher.reap_process(pid, status)
             except OSError as e:
                 if e.errno == errno.EAGAIN:
                     time.sleep(0.001)
@@ -224,6 +273,7 @@ class Arbiter(object):
         # gracefully reload watchers
         for watcher in self.iter_watchers():
             watcher.reload(graceful=graceful)
+            time.sleep(self.warmup_delay)
 
     def numprocesses(self):
         """Return the number of processes running across all watchers."""
@@ -257,7 +307,7 @@ class Arbiter(object):
             return ValueError("command name shouldn't be empty")
 
         watcher = Watcher(name, cmd, **kw)
-        watcher.initialize(self.evpub_socket)
+        watcher.initialize(self.evpub_socket, self.sockets)
         self.watchers.append(watcher)
         self._watchers_names[watcher.name.lower()] = watcher
         return watcher
@@ -281,6 +331,7 @@ class Arbiter(object):
     def start_watchers(self):
         for watcher in self.iter_watchers():
             watcher.start()
+            time.sleep(self.warmup_delay)
 
     def stop_watchers(self, stop_alive=False):
         if not self.alive:
