@@ -10,14 +10,13 @@ class Signal(Command):
         Send a signal
         =============
 
-        This command allows you to send the signal to all processes in a
-        watcher, a specific process in a watcher or its children
+        This command allows you to send a signal to all processes in a watcher,
+        a specific process in a watcher or its children.
 
         ZMQ Message
         -----------
 
-
-        To get the list of all process in the watcher::
+        To send a signal to all the processes for a watcher::
 
             {
                 "command": "signal",
@@ -32,28 +31,44 @@ class Signal(Command):
                 "command": "signal",
                 "property": {
                     "name": <name>,
-                    "process": <processid>,
+                    "pid": <processid>,
                     "signum": <signum>
             }
 
         An optionnal property "children" can be used to send the signal
-        to all the children rather than the process itself.
-
-        To send a signal to a process child:
-
-        To get the list of processes in a watcher::
+        to all the children rather than the process itself::
 
             {
                 "command": "signal",
                 "property": {
                     "name": <name>,
-                    "process": <processid>,
+                    "pid": <processid>,
                     "signum": <signum>,
-                    "pid": <pid>
+                    "children": True
             }
 
+        To send a signal to a process child::
+
+            {
+                "command": "signal",
+                "property": {
+                    "name": <name>,
+                    "pid": <processid>,
+                    "signum": <signum>,
+                    "child_pid": <childpid>,
+            }
+
+        It is also possible to send a signal to all the processes of the
+        watcher and its childs::
+
+            {
+                "command": "signal",
+                "property": {
+                    "name": <name>,
+                    "signum": <signum>,
+                    "children": True
+            }
 
-        The response return the status "ok".
 
         Command line
         ------------
@@ -66,9 +81,10 @@ class Signal(Command):
         ++++++++
 
         - <name>: the name of the watcher
-        - <process>: the process id. You can get them using the command list
         - <pid>: integer, the process id.
-        - <signum> : the signal number to send.
+        - <signum>: the signal number to send.
+        - <childpid>: the pid of a child, if any
+        - <children>: boolean, send the signal to all the children
 
         Allowed signals are:
 
@@ -80,8 +96,6 @@ class Signal(Command):
             - 22:   TTOU
             - 30:   USR1
             - 31:   USR2
-
-
     """
 
     name = "signal"
@@ -107,41 +121,43 @@ class Signal(Command):
             return self.make_message(name=args[0], signum=signum)
 
     def execute(self, arbiter, props):
-        watcher = self._get_watcher(arbiter, props['name'])
+        name = props.get('name')
+        watcher = self._get_watcher(arbiter, name)
         signum = props.get('signum')
+        pid = props.get('pid', None)
+        childpid = props.get('childpid', None)
+        children = props.get('children', False)
 
-        if 'pid' in props:
-            watcher.send_signal_child(props[1], props[2], signum)
-        elif 'process' in props:
-            process = props.get('process')
-            if props.get('children', False):
-                watcher.send_signal_children(process, signum)
+        if pid:
+            if childpid:
+                watcher.send_signal_child(pid, childpid, signum)
+            elif children:
+                watcher.send_signal_children(pid, signum)
             else:
-                watcher.send_signal(process, signum)
+                # send to the given pid
+                watcher.send_signal(pid, signum)
         else:
+            # send to all the pids for this watcher
             watcher.send_signal_processes(signum)
 
     def validate(self, props):
         super(Signal, self).validate(props)
-        if 'pid' in props and not 'process' in props:
-            raise MessageError('process ID is missing')
-
-        if props.get('children', False) and not 'process':
-            raise MessageError('process ID is missing')
 
         signum = props.get('signum')
+        if 'statspid' in props and not 'pid' in props:
+            raise ArgumentError('cannot specify childpid without pid')
+        if 'children' in props and not 'pid' in props:
+            raise ArgumentError('cannot specify children without pid')
 
         if isinstance(signum, int):
             if signum not in (signal.SIGQUIT, signal.SIGHUP, signal.SIGKILL,
                     signal.SIGTERM, signal.SIGTTIN, signal.SIGTTOU,
                     signal.SIGUSR1, signal.SIGUSR2):
                 raise MessageError('signal invalid')
+
         elif isinstance(signum, string_types):
             if signum.lower() in ('quit', 'hup', 'kill', 'term', 'ttin',
                 'ttou', 'usr1', 'usr2'):
                 props['signum'] = getattr(signal, "SIG%s" % signum.upper())
             else:
                 raise MessageError('signal invalid')
-
-        else:
-            raise MessageError('signal invalid')
