@@ -2,9 +2,7 @@ import ConfigParser
 import os
 import fnmatch
 import sys
-
-from circus.stream import FileStream
-from circus import util
+from circus import logger
 
 
 def watcher_defaults():
@@ -26,7 +24,9 @@ def watcher_defaults():
         'stderr_stream': dict(),
         'stdout_stream': dict(),
         'stream_backend': 'thread',
-        'priority': 0}
+        'priority': 0,
+        'use_sockets': False,
+        'singleton': False}
 
 
 class DefaultConfigParser(ConfigParser.ConfigParser):
@@ -49,53 +49,40 @@ def read_config(config_path):
         cfg.readfp(f)
     cfg_files_read = [config_path]
 
+    current_dir = os.path.dirname(config_path)
+
     # load included config files
     includes = []
+
+    def include_filename(filename):
+        if '*' in filename:
+            include_dir = os.path.dirname(filename)
+            if os.path.abspath(filename) != filename:
+                include_dir = os.path.join(current_dir,
+                                           os.path.dirname(filename))
+
+            wildcard = os.path.basename(filename)
+            for root, dirnames, filenames in os.walk(include_dir):
+                for filename in fnmatch.filter(filenames, wildcard):
+                    cfg_file = os.path.join(root, filename)
+                    includes.append(cfg_file)
+
+        elif os.path.isfile(filename):
+            includes.append(filename)
+
     for include_file in cfg.dget('circus', 'include', '').split():
-        includes.append(include_file)
+        include_filename(include_file)
 
     for include_dir in cfg.dget('circus', 'include_dir', '').split():
-        for root, dirnames, filenames in os.walk(include_dir):
-            for filename in fnmatch.filter(filenames, '*.ini'):
-                cfg_file = os.path.join(root, filename)
-                includes.append(cfg_file)
+        include_filename(os.path.join(include_dir, '*.ini'))
+
+    logger.debug('reading config files: %s' % includes)
 
     cfg_files_read.extend(cfg.read(includes))
 
     return cfg, cfg_files_read
 
 
-def stream_config(watcher_name, stream_conf):
-    if not stream_conf:
-        return stream_conf
-
-    if not 'class' in stream_conf:
-        # we can handle othe class there
-        if 'filename' in stream_conf:
-            obj = FileStream
-        else:
-            raise ValueError("stream configuration invalid in %r" %
-                    watcher_name)
-
-    else:
-        class_name = stream_conf.pop('class')
-        if not "." in class_name:
-            class_name = "circus.stream.%s" % class_name
-
-        obj = util.resolve_name(class_name)
-
-    # default refres_time
-    if not 'refresh_time' in stream_conf:
-        refresh_time = 0.3
-    else:
-        refresh_time = float(stream_conf.pop('refresh_time'))
-
-    # initialize stream instance
-    inst = obj(**stream_conf)
-
-    return {'stream': inst, 'refresh_time': refresh_time}
-
-
 def get_config(config_file):
     if not os.path.exists(config_file):
         sys.stderr.write("the configuration file %r does not exist\n" %
@@ -113,6 +100,11 @@ def get_config(config_file):
     config['pubsub_endpoint'] = dget('circus', 'pubsub_endpoint',
                                      'tcp://127.0.0.1:5556')
     config['stats_endpoint'] = dget('circus', 'stats_endpoint', None, str)
+    config['warmup_delay'] = dget('circus', 'warmup_delay', 0, int)
+    config['httpd'] = dget('circus', 'httpd', False, bool)
+    config['httpd_host'] = dget('circus', 'httpd_host', 'localhost', str)
+    config['httpd_port'] = dget('circus', 'httpd_port', 8080, int)
+
     stream_backend = dget('circus', 'stream_backend', 'thread')
 
     if stream_backend == 'gevent':
@@ -132,11 +124,17 @@ def get_config(config_file):
 
     config['stream_backend'] = stream_backend
 
-    # Initialize watchers & plugins to manage
+    # Initialize watchers, plugins & sockets to manage
     watchers = []
     plugins = []
+    sockets = []
 
     for section in cfg.sections():
+        if section.startswith("socket:"):
+            sock = dict(cfg.items(section))
+            sock['name'] = section.split("socket:")[-1].lower()
+            sockets.append(sock)
+
         if section.startswith("plugin:"):
             plugins.append(dict(cfg.items(section)))
 
@@ -187,19 +185,16 @@ def get_config(config_file):
                     watcher['rlimits'][limit] = int(val)
                 elif opt == 'priority':
                     watcher['priority'] = dget(section, "priority", 0, int)
+                elif opt == 'use_sockets':
+                    watcher['use_sockets'] = dget(section, "use_sockets",
+                                                  False, bool)
+                elif opt == 'singleton':
+                    watcher['singleton'] = dget(section, "singleton", False,
+                                                bool)
                 else:
                     # freeform
                     watcher[opt] = val
 
-            # second pass, parse stream conf
-            stdout_conf = watcher.get('stdout_stream', {})
-            watcher['stdout_stream'] = stream_config(watcher['name'],
-                stdout_conf)
-
-            stderr_conf = watcher.get('stderr_stream', {})
-            watcher['stderr_stream'] = stream_config(watcher['name'],
-                stderr_conf)
-
             # set the stream backend
             watcher['stream_backend'] = stream_backend
 
@@ -207,4 +202,5 @@ def get_config(config_file):
 
     config['watchers'] = watchers
     config['plugins'] = plugins
+    config['sockets'] = sockets
     return config
