@@ -58,3 +58,31 @@ class TestProcess(TestCircus):
 
         self.assertEqual(srt2ints(output['NOFILE']), wanted)
         self.assertEqual(srt2ints(output['NPROC']), wanted)
+
+    def test_comparison(self):
+        cmd = sys.executable
+        args = ['import time; time.sleep(2)', ]
+        p1 = Process('1', cmd, args=args)
+        p2 = Process('2', cmd, args=args)
+
+        self.assertTrue(p1 < p2)
+        self.assertFalse(p1 == p2)
+        self.assertTrue(p1 == p1)
+
+        p1.stop()
+        p2.stop()
+
+    def test_process_parameters(self):
+
+        # all the options passed to the process should be available by the
+        # command / process
+
+        p1 = Process('1', 'make-me-a-coffee',
+                '$(circus.wid) --type $(circus.env.type)',
+                     shell=False, spawn=False, env={'type': 'macchiato'})
+
+        self.assertEquals(['make-me-a-coffee', '1', '--type', 'macchiato'],
+                          p1.format_args())
+
+        p2 = Process('1', 'yeah $(CIRCUS.WID)', spawn=False)
+        self.assertEquals(['yeah', '1'], p2.format_args())
